require "pkernel/version"

module Pkernel
  class Error < StandardError; end
  # Your code goes here...

  module Catcher
    def method_missing(mtd,*args,&block)
      STDERR.puts "Not implemented" 
    end
  end

  #CatchAll = Catcher.new
 
  class KeyPair
    RSA_KEY_NAME = "RSA"
    DSA_KEY_NAME = "DSA"
    EC_KEY_NAME  = "EC"
    DH_KEY_NAME  = "DH"
    
    include Catcher
  end

  class Certificate
    include Catcher
    
    class Owner
      attr_accessor :name, :country, :locality, :org, :orgUnit, :serial, :csr
      attr_reader :emails, :dns_names

      def initialize
        @emails = []
        @dns_names = []
      end

      def add_email(email)
        @emails << email if not email.nil? and not email.empty?
      end

      def add_dns_names(dns)
        @dns_names << dns if not dns.nil? and not dns.empty?
      end

      def email=(email)
        @emails << email
      end

      def from_hash(hash)
        if not hash.nil?
          hash.each do |k,v|
            case k.downcase.to_sym
            when :name
              @name = v
            when :email
              add_email(v)
            when :country, :c
              @country = v
            when :org
              @org = v
            when :orgUnit, :org_unit
              @orgUnit = v
            end
          end
        end
      end
    end
    # end class Owner

    #def self.method_missing(mtd,*args,&block)
    #  if self.respond_to?(mtd)
    #    self.send(mtd,*args,&block)
    #  else
    #    super
    #  end
    #end

    #def method_missing(mtd,*args,&block)
    #  if self.respond_to?(mtd)
    #    self.send(mtd,*args,&block)
    #  else
    #    super
    #  end
    #end
    
  end
  # end Certificate

  class CSR
  end

  class CRL
  end

  class CRLReason
    include Catcher

    UNSPECIFIED = 0
    KEY_COMPROMISE = 1
    CA_COMPROMISE = 2
    AFFILIATION_CHANGED = 3
    SUPERSEDED = 4
    CESSATION_OF_OPERATION = 5
    CERTIFICATE_HOLD = 6
    REMOVE_FROM_CRL = 8
    PRIVILEGE_WITHDRAWN = 9
    AA_COMPROMISE = 10

    def self.to_string(reason)
      case reason
      when UNSPECIFIED
        "CRL reason is not specified"
      when KEY_COMPROMISE
        "Key is compromised"
      when CA_COMPROMISE
        "Certificate authority is compromised"
      when AFFILIATION_CHANGED
        "Affiliation has changed"
      when SUPERSEDED
        "Certificate is superseded"
      when CESSATION_OF_OPERATION
        "Certificate issuer has ceased operation"
      when CERTIFICATE_HOLD
        "Certificate is on hold"
      when REMOVE_FROM_CRL
        "Certificate has removed from CRL"
      when PRIVILEGE_WITHDRAWN
        "Certificate owner has privilege withdrawn"
      when AA_COMPROMISE
        "AA has compromised"
      else
        ""
      end
    end
  end

  # 
  # Identity
  # Identity is abstraction consist of keypair + certificate, stored separately
  #
  class Identity
    attr_reader :privKey, :certificate, :keystore, :chain, :key, :pubKey
    def initialize(opts)
      if self.respond_to?(:pre_init)
        pre_init(opts)
      end
      @key = opts[:key]
      @pubKey = opts[:pubKey]
      @privKey = opts[:privKey]
      @certificate = opts[:certificate]
      @keystore = opts[:keystore]
      @chain = opts[:chain]
      if self.respond_to?(:post_init)
        post_init(opts)
      end
    end

    def to_s
      res = []
      res << "Identity:"
      res << "Keypair : #{@key.nil? ? 'Not exist' : 'Exist'}"
      res << "Cert : #{@certificate.nil? ? 'Not exist' : 'Exist'}"
      res.join("\n")
    end
  end
  # end Identity
  # 

  class IdentityFactory
    # this constant is here because there is compatibility between Java and OpenSSL
    # list below is from OpenSSL website
    # Any value outside fo this list may not be opened by OpenSSL engine
    P8_PBE_MD2_DES = "PBE-MD2-DES"
    P8_PBE_MD5_DES = "PBE-MD5-DES"

    P8_PBE_SHA1_RC2_64 = "PBE-SHA1-RC2-64"
    P8_PBE_MD2_RC2_64  = "PBE-MD2-RC2-64"
    P8_PBE_MD5_RC2_64  = "PBE-MD5-RC2-64"
    P8_PBE_SHA1_DES    = "PBE-SHA1-DES"

    P8_PBE_SHA1_RC4_128 = "PBE-SHA1-RC4-128"
    P8_PBE_SHA1_RC4_40  = "PBE-SHA1-RC4-40"
    P8_PBE_SHA1_3DES    = "PBE-SHA1-3DES"
    P8_PBE_SHA1_2DES    = "PBE-SHA1-2DES"
    P8_PBE_SHA1_RC2_128 = "PBE-SHA1-RC2-128"
    P8_PBE_SHA1_RC2_40  = "PBE-SHA1-RC2-40"
  end

  class OCSP
  end

  class Rfc3161
    RESP_GRANTED = 0
    RESP_GRANTED_WITH_MODS = 1
    RESP_REJECTION = 2
    RESP_WAITING = 3
    RESP_REVOCATION_WARNING = 4
    RESP_REVOCATION_NOTIFICATION = 5
    RESP_KEY_UPDATE_WARNING = 6

    REASON_BAD_ALG = 0
    REASON_BAD_REQ = 2
    REASON_BAD_DATAFORMAT = 5
    REASON_TIME_NOT_AVAIL = 14
    REASON_UNACCEPTED_POLICY = 15
    REASON_UNACCEPTED_EXTENSION = 16
    REASON_ADD_INFO_NOT_AVAIL = 17
    REASON_SYS_FAILURE = 25
  end

  class Converter
  end
  
  class Runtime
    def Runtime.is_java_runtime?
      (RUBY_PLATFORM =~ /java/) != nil
    end
  end

  class IoUtils
  end

  #class File
  #end

  class MemBuf
  end
end



# auto load related libraries?
#if Pkernel::Runtime.is_java_runtime?
#  
#  dep = Gem::Dependency.new('pkernel_jce')
#  dd = dep.matching_specs.max_by(&:version)
#  if dd.nil?
#    begin
#      # gcrypto_jce not installed!
#      Gem.install 'pkernels_jce', dd.requirement
#    rescue Exception => ex
#      # silently ignore...
#    end
#  else
#    require 'pkernel_jce'
#  end
#
#else
#  
#  dep = Gem::Dependency.new('pkernel_openssl')
#  dd = dep.matching_specs.max_by(&:version)
#  if dd.nil?
#    begin
#      # gcrypto_jce not installed!
#      Gem.install 'pkernel_openssl', dd.requirement
#    rescue Exception
#      # silently ignore...
#    end
#  else
#    require 'pkernel_openssl'
#  end
#
#end


